#!/usr/bin/ruby

require "webrick"
require "webrick/httpproxy"
require "json"

server = WEBrick::HTTPProxyServer.new(:BindAddress => "0.0.0.0", :Port => 8180)

server.mount_proc "/sync" do |req, res|
  user = req.query["user"].gsub(" ", "+")
  repo = req.query["repo"]
  branch = req.query["branch"]
  is_sync = req.query["sync"] == "true"

  path = File.join(ENV.fetch("PROJECT_ROOT"), user, repo)
  if Dir.exist?(path)
    bin_path = File.join(File.dirname(__FILE__), "conglomerate.rb")
    pid = spawn("#{bin_path} #{'-s' if is_sync} #{branch} -p #{path}")
    Process.detach(pid)
    res.status = 200
  else
    res.status = 400
  end
end

server.mount_proc "/progress" do |req, res|
  user = req.query["user"].gsub(" ", "+")
  repo = req.query["repo"]
  branch = req.query["branch"]

  path = File.join(ENV.fetch("PROJECT_ROOT"), user, repo)
  if Dir.exist?(path)
    resp = {
      commits: commit_count(path),
      indexed: indexed_count(path, branch)
    }
    res.status = 200
    res.body = resp.to_json
  else
    res.status = 400
  end
end

def commit_count(project_root)
  git_dir = File.join(project_root, ".git")
  `git --git-dir=#{git_dir} log --graph --oneline | grep '^\*' | wc -l`.to_i
end

def indexed_count(project_root, branch)
  branch = branch.gsub(/^remotes\/origin\//, "")
  branch_lock_path = File.join(project_root, ".gitolemy", "branches", "#{branch}.lock")
  `wc -l #{branch_lock_path}`
    .split(" ")
    .first
    .to_i
end

trap("INT") { server.shutdown }
server.start
