require "active_support/core_ext/object"

require_relative "commit_stats"

class RiskAnalyzer
  # TODO: Add config to match test files
  def analyze(file_manager, commit)
    return if commit.nil?

    files = files_for_diff(commit.file_diffs, file_manager)

    CommitStats.link_mutations!(files, commit.changes, :change)
    CommitStats.link_mutations!(files, commit.movements, :movement)
    line_diffs = CommitStats.link_diffs!(files, commit.file_diffs)

    score(line_diffs)
  end

  # TODO: Maybe include function score per line.
  def score(line_diffs)
    (line_diffs[:insertions] + line_diffs[:deletions])
      .reduce(0) do |acc, line|
        penalty = Line.relevant_change?(line) ? Line.score(line) : 0
        penalty /= 3 if line.has_key?(:movement)
        acc += penalty
        acc
      end
  end

  private

  def files_for_diff(file_diffs, file_manager)
    file_diffs.reduce({a: {}, b: {}}) do |acc, (b_path, file_diff)|
      file = {
        a_file: file_manager.vfs[file_diff.a_file_name] || load_file(file_diff),
        b_file: file_manager.vfs[file_diff.b_file_name],
        a_path: file_diff.a_file_name,
        b_path: file_diff.b_file_name,
        changes: [],
        movements: []
      }
      acc[:a][file[:a_path]] = file
      acc[:b][file[:b_path]] = file
      acc
    end
  end

  def load_file(file_diff)
    Cache
      .read_object(file_diff.a_file_id)
      .deep_symbolize_keys
  end
end
