require "openssl"
require "json"


class SecureFileStore
  def initialize(key)
    @key = unhex(key)
  end

  def write_file(data, file_path)
    iv, data = encrypt(@key, data)
    File.open(file_path, "wb") { |file| file.write(data) }
  end

  def read_file(iv, file_path)
    data = File.read(file_path)
    decrypt(@key, unhex(iv), data)
  end

  def write_settings(settings, dir=".gitolemy")
    iv, data = encrypt(@key, settings.to_json())
    file_path = File.join(dir, "config-#{hex(iv)}")
    File.open(file_path, "wb") { |file| file.write(data) }
  end

  def read_settings(dir=".gitolemy")
    file_path = Dir[File.join(dir, "config-*")].first
    iv = File.basename(file_path).gsub(/^config-/, "")
    JSON.parse(read_file(iv, file_path))
  end

  private

  def hex(iv)
    iv.unpack("H*").first
  end

  def unhex(iv)
    iv
      .scan(/../)
      .map { |x| x.hex }
      .pack("c*")
  end

  def encrypt(private_key, data)
    cipher = OpenSSL::Cipher::AES.new(256, :CBC)
    cipher.encrypt
    cipher.key = private_key
    iv = cipher.random_iv
    out = cipher.update(data) + cipher.final
    [iv, out]
  end

  def decrypt(private_key, iv, data)
    cipher = OpenSSL::Cipher::AES.new(256, :CBC)
    cipher.decrypt
    cipher.key = private_key
    cipher.iv = iv
    cipher.update(data) + cipher.final
  end
end
