require_relative "cache"

module Store
  def Store.read(set, key)
    set[key]
  end

  def Store.write(set, key, value)
    set[key] = value
    key
  end

  def Store.cache(cache_key, new, old)
    Cache.write(cache_key, new, old)
  end

  module Commit
    @@commits = {}
    def Commit.[](commit_id)
      ::Store::read(@@commits, commit_id)
    end

    def Commit.index(commit)
      ::Store::write(@@commits, commit[:commit_id], commit)
    end

    def Commit.cache(old={})
      ::Store::cache("commits", @@commits, Cache.read("commits", {}))
    end
  end

  module Line
    @@lines = {}
    def Line.[](line_id)
      ::Store::read(@@lines, line_id)
    end

    def Line.index(line)
      ::Store::write(@@lines, line[:line_id], line)
    end

    def Line.cache()
      ::Store::cache("lines", @@lines, Cache.read("lines", {}))
    end
  end

  module Issue
    @@issues = {}
    def Issue.[](issue_id)
      ::Store::read(@@issues, issue_id)
    end

    def Issue.index(issue)
      ::Store::write(@@issues, issue[:issue_id], issue)
    rescue
      issue
    end

    def Issue.cache(old={})
      ::Store::cache("issues", @@issues, old)
    end
  end

  module Bug
    @@bugs = {}
    def Bug.[](bug_id)
      ::Store::read(@@bugs, bug_id)
    end

    def Bug.index(bug)
      ::Store::write(@@bugs, bug[:bug_id], bug)
    rescue
      bug
    end

    def Bug.cache(old={})
      ::Store::cache("bugs", @@bugs, old)
    end
  end


  module Error
    @@errors = {}
    def Error.[](error_id)
      ::Store::read(@@errors, error_id)
    end

    def Error.index(error)
      ::Store::write(@@errors, error[:error_id], error)
    end

    def Error.cache(old={})
      ::Store::cache("errors", @@errors, old)
    end
  end
end
